require "spec_helper"

describe ActiveRecord::Coders::Encryptors::AES do

  it "should create blank" do
    DbAes.create
  end

  context "binary" do

    it "should handle nil values" do
      aes256 = DbAes.new
      aes256.binary_data = nil
      aes256.save
      aes256.reload.binary_data.should == nil
    end

    it "should treat empty string as nil" do
      aes256 = DbAes.new
      aes256.binary_data = ""
      aes256.save
      aes256.reload.binary_data.should == nil
    end

    it "should (de)compress" do
      aes256 = DbAes.new
      aes256.binary_data = read_fixture("lipsum.txt")
      aes256.save
      aes256.binary_data.should == read_fixture("lipsum.txt")
      aes256.binary_data.bytesize.should == 3640
      aes256.reload.instance_variable_get("@attributes")["binary_data"].
        value.bytesize.should == 3664
      aes256.reload.instance_variable_get("@attributes")["binary_data"].
        should_not == aes256.binary_data
    end
  end

  context "text" do

    it "should handle nil values" do
      aes256 = DbAes.new
      aes256.text_data = nil
      aes256.save
      aes256.reload.text_data.should == nil
    end

    it "should treat empty string as nil" do
      aes256 = DbAes.new
      aes256.text_data = ""
      aes256.save
      aes256.reload.text_data.should == nil
    end

    it "should (de)compress" do
      aes256 = DbAes.new
      aes256.text_data = read_fixture("lipsum.txt")
      aes256.save
      aes256.text_data.should == read_fixture("lipsum.txt")
      aes256.text_data.bytesize.should == 3640
      aes256.reload.instance_variable_get("@attributes")["text_data"].
        value.bytesize.should == 4970
      aes256.reload.instance_variable_get("@attributes")["text_data"].
        should_not == aes256.text_data
    end
  end
end
