require "spec_helper"

describe ActiveRecord::Coders::Serializers::Marshal do

  context "binary" do

    it "should create blank" do
      DbMarshal.create
    end

    it "should handle nil values" do
      marshal = DbMarshal.new
      marshal.binary_data = nil
      marshal.save
      marshal.reload.binary_data.should == nil
    end

    it "should (de)serialize" do
      marshal = DbMarshal.new
      marshal.binary_data = { some: "data", with: %w[a r r a y] }
      marshal.save
      marshal.reload.binary_data.should == { some: "data", with: %w[a r r a y] }
      marshal.reload.instance_variable_get("@attributes")["binary_data"]
      .value.bytesize.should == 70
    end

    it "should raise exception when receiving invalid data" do
      marshal = DbMarshal.new
      marshal.binary_data = proc {}
      expect { marshal.save }.to raise_error(TypeError)
      expect { marshal.reload }.to raise_error(ActiveRecord::RecordNotFound)
    end
  end

  context "text" do

    it "should create blank" do
      DbMarshal.create
    end

    it "should handle nil values" do
      marshal = DbMarshal.new
      marshal.text_data = nil
      marshal.save
      marshal.reload.text_data.should == nil
    end

    it "should (de)serialize" do
      marshal = DbMarshal.new
      marshal.text_data = { some: "data", with: %w[a r r a y] }
      marshal.save
      marshal.reload.text_data.should == { some: "data", with: %w[a r r a y] }
      marshal.reload.instance_variable_get("@attributes")["text_data"]
      .value.bytesize.should == 98
    end

    it "should raise exception when receiving invalid data" do
      marshal = DbMarshal.new
      marshal.text_data = proc {}
      expect { marshal.save }.to raise_error(TypeError)
      expect { marshal.reload }.to raise_error(ActiveRecord::RecordNotFound)
    end
  end
end
