require "spec_helper"

describe ActiveRecord::Coders::Serializers::MessagePack do

  context "binary" do

    it "should create blank" do
      DbMessagePack.create
    end

    it "should handle nil values" do
      msgpack = DbMessagePack.new
      msgpack.binary_data = nil
      msgpack.save
      msgpack.reload.binary_data.should == nil
    end

    it "should (de)serialize" do
      msgpack = DbMessagePack.new
      msgpack.binary_data = { some: "data", with: %w[a r r a y] }
      msgpack.save
      msgpack.reload.binary_data.should ==
        { "some" => "data", "with" => %w[a r r a y] }
      msgpack.reload.instance_variable_get("@attributes")["binary_data"]
      .value.bytesize.should == 27
    end

    it "should raise exception when receiving invalid data" do
      msgpack = DbMessagePack.new
      msgpack.binary_data = proc {}
      expect { msgpack.save }.to raise_error(NoMethodError)
      expect { msgpack.reload }.to raise_error(ActiveRecord::RecordNotFound)
    end
  end

  context "text" do

    it "should create blank" do
      DbMessagePack.create
    end

    it "should handle nil values" do
      msgpack = DbMessagePack.new
      msgpack.text_data = nil
      msgpack.save
      msgpack.reload.text_data.should == nil
    end

    it "should (de)serialize" do
      msgpack = DbMessagePack.new
      msgpack.text_data = { some: "data", with: %w[a r r a y] }
      msgpack.save
      msgpack.reload.text_data.should ==
        { "some" => "data", "with" => %w[a r r a y] }
      msgpack.reload.instance_variable_get("@attributes")["text_data"]
      .value.bytesize.should == 37
    end

    it "should raise exception when receiving invalid data" do
      msgpack = DbMessagePack.new
      msgpack.text_data = proc {}
      expect { msgpack.save }.to raise_error(NoMethodError)
      expect { msgpack.reload }.to raise_error(ActiveRecord::RecordNotFound)
    end
  end
end
