require "spec_helper"

describe ActiveRecord::Coders::Serializers::Oj do

  it "should create blank" do
    DbOj.create
  end

  context "default, object" do

    it "should handle nil values" do
      oj = DbOj.new
      oj.default_data = nil
      oj.object_data = nil
      oj.save
      oj.reload.default_data.should == nil
      oj.reload.object_data.should == nil
    end

    it "should (de)serialize" do
      data = OpenStruct.new(
        name: "Michael",
        colors: %w[red white blue]
      )

      oj = DbOj.new
      oj.default_data = data
      oj.object_data = data
      oj.save
      oj.reload.default_data.should == data
      oj.reload.object_data.should == data
    end

    it "should replace unknown types with JSON null values" do
      oj = DbOj.new
      oj.default_data = { "example" => 1, "test" => proc{} }
      oj.object_data = { "example" => 1, "test" => proc{} }
      oj.save
      oj.reload.default_data.should == { "example" => 1, "test" => nil }
      oj.reload.object_data.should == { "example" => 1, "test" => nil }
    end
  end

  context "strict" do

    it "should handle nil values" do
      oj = DbOj.new
      oj.strict_data = nil
      oj.save
      oj.reload.strict_data.should == nil
    end

    it "should (de)serialize" do
      oj = DbOj.new
      oj.strict_data = { "some" => "data", "with" => %w[a r r a y] }
      oj.save
      oj.reload.strict_data.should == { "some" => "data", "with" => %w[a r r a y] }
    end

    it "should raise exception when receiving invalid data" do
      oj = DbOj.new
      oj.strict_data = { some:  "data", with: %w[a r r a y] }
      expect { oj.save }.to raise_error(TypeError)
      expect { oj.reload }.to raise_error(ActiveRecord::RecordNotFound)
    end
  end

  context "compat" do

    it "should handle nil values" do
      oj = DbOj.new
      oj.compat_data = nil
      oj.save
      oj.reload.compat_data.should == nil
    end

    it "should (de)serialize" do
      oj = DbOj.new
      oj.compat_data = { some:  "data", with: %w[a r r a y] }
      oj.save
      oj.reload.compat_data.should == { "some" => "data", "with" => %w[a r r a y] }
    end
  end

  context "null" do

    it "should handle nil values" do
      oj = DbOj.new
      oj.null_data = nil
      oj.save
      oj.reload.null_data.should == nil
    end

    it "should (de)serialize" do
      oj = DbOj.new
      oj.null_data = { "some" => "data", "with" => %w[a r r a y] }
      oj.save
      oj.reload.null_data.should == { "some" => "data", "with" => %w[a r r a y] }
    end

    it "should replace unknown types with JSON null values" do
      oj = DbOj.new
      oj.null_data = { "example" => 1, "test" => proc{} }
      oj.save
      oj.reload.null_data.should == { "example" => 1, "test" => nil }
    end
  end
end
